package com.lzmispt.data.resource.service.report.impl;

package .tmp;

import com.alibaba.csp.sentinel.util.StringUtil;
import com.alibaba.fastjson.JSON;
import com.lzmispt.core.dao.database.impl.BaseDaoImpl;
import com.lzmispt.core.entity.dto.authorize.UserInfoDTO;
import com.lzmispt.core.entity.dto.listdeletemodel.ListDeleteModelDTO;
import com.lzmispt.core.entity.dto.querymodel.QueryModelDTO;
import com.lzmispt.core.entity.vo.user.UserVO;
import com.lzmispt.core.service.listdelete.ListDeleteUtil;
import com.lzmispt.core.service.query.IQuery;
import com.lzmispt.data.center.entity.dto.resource.report.ListDeleteReportDTO;
import com.lzmispt.data.center.entity.dto.resource.report.QueryReportListDTO;
import com.lzmispt.data.center.entity.dto.resource.report.QueryReportTreeDTO;
import com.lzmispt.data.center.entity.dto.resource.report.ReportConfigDTO;
import com.lzmispt.data.center.entity.dto.resource.report.SaveReportDTO;
import com.lzmispt.data.center.entity.dto.resource.report.UReportDTO;
import com.lzmispt.data.center.entity.dto.treesortmodel.DCTreeSortModelDTO;
import com.lzmispt.data.center.entity.enums.StatusCodeEnum;
import com.lzmispt.data.center.entity.vo.resource.report.ReportTreeVO;
import com.lzmispt.data.center.entity.vo.resource.report.ReportVO;
import com.lzmispt.data.resource.client.CoreClient;
import com.lzmispt.data.resource.client.ReportClient;
import com.lzmispt.data.resource.dao.report.IReportDao;
import com.lzmispt.data.resource.entity.po.report.ReportPO;
import com.lzmispt.data.resource.service.desensitize.IDataDesensitizeService;
import com.lzmispt.data.resource.service.metadata.impl.ApplyDataBrowseImpl;
import com.lzmispt.data.resource.service.report.IReportService;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.dcbase.util.SimpleCacheUtil;
import com.lzmispt.dcbase.util.TokenUtil;
import com.lzmispt.framework.entity.entity.ResultEntity;
import com.lzmispt.framework.utils.WorkerId;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

@Service
@RefreshScope
public class ReportServiceImpl implements IReportService {
  private static Long MANAGE_TABLE_PARENT = Long.valueOf(210000000000000000L);
  
  @Value("${data-center.data-mode:mysql}")
  private String dbType;
  
  @Autowired
  private CoreClient coreClient;
  
  @Autowired
  private ReportClient reportClient;
  
  @Autowired
  private IReportDao reportDao;
  
  @Autowired
  private BaseDaoImpl baseDao;
  
  @Autowired
  private IQuery query;
  
  @Autowired
  private IDataDesensitizeService dataDesensitizeService;
  
  @Autowired
  private ApplyDataBrowseImpl applyDataBrowse;
  
  public ResultEntity<List<ReportTreeVO>> getReportDirectoryTree(QueryReportTreeDTO param, String tokenId) {
    List<ReportPO> childReportDirList;
    if (DCConst.TRUE.equals(param.getIsLazyLoading())) {
      childReportDirList = this.reportDao.selectReportDirByParentId(param.getParentId());
      if (!StringUtil.isEmpty(param.getName()))
        childReportDirList = (List<ReportPO>)childReportDirList.stream().filter(item -> (item.getName() != null && item.getName().toLowerCase().contains(param.getName().toLowerCase()))).collect(Collectors.toList()); 
    } else if (!StringUtil.isEmpty(param.getName())) {
      childReportDirList = this.reportDao.recursiveSelectReportDirTreeByName(param.getName());
    } else {
      childReportDirList = this.reportDao.recursiveSelectReportDirTreeByParentId(param.getParentId());
    } 
    List<ReportTreeVO> resultTree = new ArrayList<>();
    recursiveBuildReportTree(param.getParentId(), childReportDirList, resultTree);
    return ResultEntity.successMessage(resultTree);
  }
  
  public ResultEntity<List<ReportVO>> getReportList(QueryReportListDTO param, String tokenId) {
    boolean isEscapeStr = false;
    if (this.dbType.equalsIgnoreCase("sqlserver"))
      isEscapeStr = true; 
    String whereSql = (String)this.query.getWhereSqlStr((QueryModelDTO)param, this.dbType, true, isEscapeStr).getObj();
    if (!StringUtil.isEmpty(whereSql) && param.getFilterModel() != null)
      whereSql = " and " + whereSql; 
    Map<String, Object> queryParam = new HashMap<>();
    queryParam.put("parentId", param.getParentId());
    queryParam.put("sql", whereSql);
    List<ReportPO> reportList = this.reportDao.selectReportList(queryParam);
    List<ReportVO> result = new ArrayList<>(reportList.size());
    Map<Long, UserVO> userCache = new HashMap<>();
    for (ReportPO reportPO : reportList)
      result.add(getReportResultByPO(reportPO, userCache, tokenId)); 
    return ResultEntity.successMessage(result);
  }
  
  public ResultEntity<Boolean> checkReportDuplicate(SaveReportDTO param, String tokenId) {
    if (isNameDuplicate(param)) {
      StatusCodeEnum nameExist = DCConst.ReportType.DIRECTORY.equals(param.getType()) ? StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NAME_EXIST : StatusCodeEnum.DATA_RESOURCE_REPORT_NAME_EXIST;
      return ResultEntity.errorMessage(Integer.valueOf(nameExist.getCode()), nameExist.getMessage());
    } 
    return ResultEntity.successMessage(Boolean.valueOf(false));
  }
  
  public ResultEntity<ReportVO> addReport(SaveReportDTO param, String tokenId) {
    if (isNameDuplicate(param)) {
      StatusCodeEnum nameExist = DCConst.ReportType.DIRECTORY.equals(param.getType()) ? StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NAME_EXIST : StatusCodeEnum.DATA_RESOURCE_REPORT_NAME_EXIST;
      return ResultEntity.errorMessage(Integer.valueOf(nameExist.getCode()), nameExist.getMessage());
    } 
    ReportPO insert = new ReportPO();
    UserInfoDTO userInfo = TokenUtil.getUserInfoByTokenId(tokenId);
    BeanUtils.copyProperties(param, insert);
    insert.setUserIdRecord(userInfo.getUserId());
    insert.setOrgIdRecord(userInfo.getOrganizationId());
    insert.setId((Long)WorkerId.getNextId(Long.class));
    insert.setOrder(getNextOrder(param));
    insert.setCreateTime(new Date());
    insert.setUpdateTime(new Date());
    insert.setIsDelete(DCConst.FALSE);
    if (DCConst.ReportType.REPORT.equals(insert.getType())) {
      UReportDTO uReport = (UReportDTO)this.reportClient.insertReport(buildUReport(insert, param.getReportConfig()), tokenId).getObj();
      insert.setUId(uReport.getId());
      param.getReportConfig().setTitle(insert.getName());
      param.getReportConfig().setApplyAddress(uReport.getCode());
      insert.setConfig(JSON.toJSONString(param.getReportConfig()));
    } 
    this.baseDao.save(insert);
    ReportVO result = getReportResultByPO(insert, tokenId);
    return ResultEntity.successMessage(result);
  }
  
  private UReportDTO buildUReport(ReportPO reportPO, ReportConfigDTO config) {
    UReportDTO uReport = new UReportDTO();
    uReport.setId(reportPO.getUId());
    uReport.setUserIdRecord(reportPO.getUserIdRecord());
    uReport.setOrgIdRecord(reportPO.getOrgIdRecord());
    return uReport;
  }
  
  public ResultEntity<ReportVO> renameReport(SaveReportDTO param, String tokenId) {
    ReportPO beforeUpdate = (ReportPO)this.baseDao.load(param.getId(), ReportPO.class);
    if (beforeUpdate == null) {
      StatusCodeEnum notExist = DCConst.ReportType.DIRECTORY.equals(param.getType()) ? StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NOT_EXIST : StatusCodeEnum.DATA_RESOURCE_REPORT_NOT_EXIST;
      return ResultEntity.errorMessage(Integer.valueOf(notExist.getCode()), notExist.getMessage());
    } 
    param.setParentId(beforeUpdate.getParentId());
    param.setType(beforeUpdate.getType());
    if (isNameDuplicate(param)) {
      StatusCodeEnum nameExist = DCConst.ReportType.DIRECTORY.equals(param.getType()) ? StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NAME_EXIST : StatusCodeEnum.DATA_RESOURCE_REPORT_NAME_EXIST;
      return ResultEntity.errorMessage(Integer.valueOf(nameExist.getCode()), nameExist.getMessage());
    } 
    ReportPO update = new ReportPO();
    update.setId(param.getId());
    update.setName(param.getName());
    update.setUpdateTime(new Date());
    if (DCConst.ReportType.REPORT.equals(beforeUpdate.getType())) {
      ReportConfigDTO configDTO = (ReportConfigDTO)JSON.parseObject(beforeUpdate.getConfig(), ReportConfigDTO.class);
      configDTO.setTitle(update.getName());
      update.setConfig(JSON.toJSONString(configDTO));
    } 
    this.baseDao.updateSelective(update);
    ReportPO afterUpdate = (ReportPO)this.baseDao.load(param.getId(), ReportPO.class);
    ReportVO result = getReportResultByPO(afterUpdate, tokenId);
    return ResultEntity.successMessage(result);
  }
  
  public ResultEntity<ReportVO> saveConfig(SaveReportDTO param, String tokenId) {
    ReportPO beforeUpdate = (ReportPO)this.baseDao.load(param.getId(), ReportPO.class);
    if (beforeUpdate == null) {
      StatusCodeEnum notExist = DCConst.ReportType.DIRECTORY.equals(param.getType()) ? StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NOT_EXIST : StatusCodeEnum.DATA_RESOURCE_REPORT_NOT_EXIST;
      return ResultEntity.errorMessage(Integer.valueOf(notExist.getCode()), notExist.getMessage());
    } 
    ReportPO update = new ReportPO();
    update.setId(param.getId());
    param.getReportConfig().setTitle(beforeUpdate.getName());
    update.setConfig(JSON.toJSONString(param.getReportConfig()));
    update.setUpdateTime(new Date());
    this.baseDao.updateSelective(update);
    ReportPO afterUpdate = (ReportPO)this.baseDao.load(param.getId(), ReportPO.class);
    ReportVO result = getReportResultByPO(afterUpdate, tokenId);
    return ResultEntity.successMessage(result);
  }
  
  public ResultEntity<ReportVO> moveReport(SaveReportDTO param, String tokenId) {
    ReportPO beforeUpdate = (ReportPO)this.baseDao.load(param.getId(), ReportPO.class);
    if (beforeUpdate == null) {
      StatusCodeEnum notExist = DCConst.ReportType.DIRECTORY.equals(param.getType()) ? StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NOT_EXIST : StatusCodeEnum.DATA_RESOURCE_REPORT_NOT_EXIST;
      return ResultEntity.errorMessage(Integer.valueOf(notExist.getCode()), notExist.getMessage());
    } 
    param.setType(beforeUpdate.getType());
    param.setName(beforeUpdate.getName());
    if (isNameDuplicate(param)) {
      StatusCodeEnum nameExist = DCConst.ReportType.DIRECTORY.equals(param.getType()) ? StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NAME_EXIST : StatusCodeEnum.DATA_RESOURCE_REPORT_NAME_EXIST;
      return ResultEntity.errorMessage(Integer.valueOf(nameExist.getCode()), nameExist.getMessage());
    } 
    if (Objects.equals(param.getParentId(), beforeUpdate.getParentId())) {
      ReportVO reportVO = new ReportVO();
      BeanUtils.copyProperties(beforeUpdate, reportVO);
      return ResultEntity.successMessage(reportVO);
    } 
    ReportPO update = new ReportPO();
    update.setId(param.getId());
    update.setParentId(param.getParentId());
    update.setOrder(getNextOrder(param));
    update.setUpdateTime(new Date());
    this.reportDao.moveReport(update);
    ReportPO afterUpdate = (ReportPO)this.baseDao.load(param.getId(), ReportPO.class);
    ReportVO result = getReportResultByPO(afterUpdate, tokenId);
    return ResultEntity.successMessage(result);
  }
  
  public ResultEntity<Boolean> moveReportDir(DCTreeSortModelDTO dragDTO, String tokenId) {
    ReportPO dragPO = (ReportPO)this.baseDao.load(dragDTO.getDragId(), ReportPO.class);
    if (dragPO == null)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NOT_EXIST.getCode()), StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NOT_EXIST.getMessage()); 
    if (!dragDTO.getSameLevel().booleanValue()) {
      SaveReportDTO checkName = new SaveReportDTO();
      checkName.setId(dragPO.getId());
      checkName.setName(dragPO.getName());
      checkName.setType(dragPO.getType());
      checkName.setParentId(dragDTO.getParentId());
      if (isNameDuplicate(checkName))
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NAME_EXIST.getCode()), StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NAME_EXIST.getMessage()); 
      if (dragDTO.getParentId() != null) {
        ReportPO parentPO = (ReportPO)this.baseDao.load(dragDTO.getParentId(), ReportPO.class);
        if (parentPO == null)
          return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.TREE_SORT_MODEL_TARGET_NODE_NOT_EXIST.getCode()), StatusCodeEnum.TREE_SORT_MODEL_TARGET_NODE_NOT_EXIST.getMessage()); 
      } 
    } 
    if (CollectionUtils.isEmpty(dragDTO.getSortIds()))
      return dragNotExpanded(dragDTO); 
    return dragExpanded(dragDTO, dragPO);
  }
  
  public ResultEntity<ReportVO> copyReport(SaveReportDTO param, String tokenId) {
    ReportPO originReport = (ReportPO)this.baseDao.load(param.getId(), ReportPO.class);
    if (originReport == null) {
      StatusCodeEnum notExist = DCConst.ReportType.DIRECTORY.equals(param.getType()) ? StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NOT_EXIST : StatusCodeEnum.DATA_RESOURCE_REPORT_NOT_EXIST;
      return ResultEntity.errorMessage(Integer.valueOf(notExist.getCode()), notExist.getMessage());
    } 
    param.setId(null);
    param.setType(originReport.getType());
    if (isNameDuplicate(param)) {
      StatusCodeEnum nameExist = DCConst.ReportType.DIRECTORY.equals(param.getType()) ? StatusCodeEnum.DATA_RESOURCE_REPORT_DIR_NAME_EXIST : StatusCodeEnum.DATA_RESOURCE_REPORT_NAME_EXIST;
      return ResultEntity.errorMessage(Integer.valueOf(nameExist.getCode()), nameExist.getMessage());
    } 
    ReportPO copy = new ReportPO();
    BeanUtils.copyProperties(param, copy);
    UserInfoDTO userInfo = TokenUtil.getUserInfoByTokenId(tokenId);
    copy.setUserIdRecord(userInfo.getUserId());
    copy.setOrgIdRecord(userInfo.getOrganizationId());
    copy.setUId(originReport.getUId());
    ReportConfigDTO configDTO = (ReportConfigDTO)JSON.parseObject(originReport.getConfig(), ReportConfigDTO.class);
    configDTO.setTitle(copy.getName());
    UReportDTO uReport = (UReportDTO)this.reportClient.copyReport(buildUReport(copy, configDTO), tokenId).getObj();
    copy.setUId(uReport.getId());
    configDTO.setApplyAddress(uReport.getCode());
    copy.setConfig(JSON.toJSONString(configDTO));
    copy.setId((Long)WorkerId.getNextId(Long.class));
    copy.setOrder(getNextOrder(param));
    copy.setCreateTime(new Date());
    copy.setUpdateTime(new Date());
    copy.setIsDelete(DCConst.FALSE);
    this.baseDao.save(copy);
    ReportVO result = getReportResultByPO(copy, tokenId);
    return ResultEntity.successMessage(result);
  }
  
  public ResultEntity<Boolean> deleteReport(Long reportId, String tokenId) {
    ReportPO beforeDelete = (ReportPO)this.baseDao.load(reportId, ReportPO.class);
    if (beforeDelete == null)
      return ResultEntity.successMessage(Boolean.valueOf(true)); 
    if (DCConst.ReportType.REPORT.equals(beforeDelete.getType())) {
      this.baseDao.delete(reportId, ReportPO.class);
      this.reportClient.deleteBatch(Collections.singletonList(beforeDelete.getUId()), tokenId);
      return ResultEntity.successMessage(Boolean.valueOf(true));
    } 
    List<ReportPO> childReportDirList = this.reportDao.selectReportDirByParentId(reportId);
    if (!CollectionUtils.isEmpty(childReportDirList))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_REPORT_EXIST_CHILD_DIR.getCode()), StatusCodeEnum.DATA_RESOURCE_REPORT_EXIST_CHILD_DIR.getMessage()); 
    List<ReportPO> childReportList = this.reportDao.selectReportByParentId(reportId);
    if (!CollectionUtils.isEmpty(childReportList))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_REPORT_EXIST_CHILD_REPORT.getCode()), StatusCodeEnum.DATA_RESOURCE_REPORT_EXIST_CHILD_REPORT.getMessage()); 
    this.baseDao.delete(reportId, ReportPO.class);
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  public ResultEntity<Map<String, List<String>>> listDeleteReport(ListDeleteReportDTO param, String tokenId) {
    String whereSql = (String)ListDeleteUtil.getListDeleteWhere((ListDeleteModelDTO)param, this.dbType).getObj();
    if (!StringUtil.isEmpty(whereSql))
      whereSql = " and " + whereSql; 
    Map<String, Object> listDeleteParam = new HashMap<>();
    listDeleteParam.put("parentId", param.getParentId());
    listDeleteParam.put("condition", whereSql);
    List<ReportPO> reportPOList = this.reportDao.selectReportListByCondition(listDeleteParam);
    Map<String, List<String>> result = new HashMap<>();
    List<String> successList = new ArrayList<>();
    for (ReportPO reportPO : reportPOList) {
      this.baseDao.delete(reportPO.getId(), ReportPO.class);
      successList.add(String.valueOf(reportPO.getId()));
    } 
    List<Long> uIds = (List<Long>)reportPOList.stream().map(ReportPO::getUId).filter(Objects::nonNull).collect(Collectors.toList());
    if (!uIds.isEmpty())
      this.reportClient.deleteBatch(uIds, tokenId); 
    result.put("successList", successList);
    result.put("failedList", new ArrayList<>());
    return ResultEntity.successMessage(String.format(StatusCodeEnum.DATA_RESOURCE_DELETE_MSG.getMessage(), new Object[] { Integer.valueOf(successList.size()), Integer.valueOf(0) }), result);
  }
  
  public ResultEntity<ReportConfigDTO> getReportConfig(Long uid) {
    ReportPO reportPO = this.reportDao.selectReportByUid(uid);
    if (reportPO == null)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_REPORT_NOT_EXIST.getCode()), StatusCodeEnum.DATA_RESOURCE_REPORT_NOT_EXIST.getMessage()); 
    return ResultEntity.successMessage(JSON.parseObject(reportPO.getConfig(), ReportConfigDTO.class));
  }
  
  private void recursiveBuildReportTree(Long parentId, List<ReportPO> allReportDirList, List<ReportTreeVO> resultTree) {
    List<ReportPO> directChild = (List<ReportPO>)allReportDirList.stream().filter(item -> Objects.equals(item.getParentId(), parentId)).collect(Collectors.toList());
    for (ReportPO reportPO : directChild) {
      ReportTreeVO reportTreeVO = new ReportTreeVO();
      BeanUtils.copyProperties(reportPO, reportTreeVO);
      reportTreeVO.setChildren(new ArrayList());
      resultTree.add(reportTreeVO);
      recursiveBuildReportTree(reportPO.getId(), allReportDirList, reportTreeVO.getChildren());
    } 
  }
  
  private ReportVO getReportResultByPO(ReportPO reportPO, String tokenId) {
    ReportVO result = new ReportVO();
    BeanUtils.copyProperties(reportPO, result);
    if (reportPO.getUserIdRecord() != null) {
      UserVO userVO = (UserVO)this.coreClient.getUserVOById(reportPO.getUserIdRecord(), tokenId).getObj();
      Optional.<UserVO>ofNullable(userVO).ifPresent(item -> result.setCreator(item.getUserName()));
    } 
    return result;
  }
  
  private ReportVO getReportResultByPO(ReportPO reportPO, Map<Long, UserVO> userCache, String tokenId) {
    ReportVO result = new ReportVO();
    BeanUtils.copyProperties(reportPO, result);
    if (reportPO.getUserIdRecord() != null) {
      UserVO userVO = (UserVO)SimpleCacheUtil.cacheGet(userCache, reportPO.getUserIdRecord(), () -> (UserVO)this.coreClient.getUserVOById(reportPO.getUserIdRecord(), tokenId).getObj());
      Optional.<UserVO>ofNullable(userVO).ifPresent(item -> result.setCreator(item.getUserName()));
    } 
    return result;
  }
  
  private ResultEntity<Boolean> dragNotExpanded(DCTreeSortModelDTO dragDTO) {
    SaveReportDTO getOrder = new SaveReportDTO();
    getOrder.setType(DCConst.ReportType.DIRECTORY);
    getOrder.setParentId(dragDTO.getParentId());
    ReportPO update = new ReportPO();
    update.setId(dragDTO.getDragId());
    update.setParentId(dragDTO.getParentId());
    update.setOrder(getNextOrder(getOrder));
    update.setUpdateTime(new Date());
    this.reportDao.moveReport(update);
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  private ResultEntity<Boolean> dragExpanded(DCTreeSortModelDTO dragDTO, ReportPO dragPO) {
    List<ReportPO> children = this.reportDao.selectReportDirByParentId(dragDTO.getParentId());
    Map<Long, ReportPO> childrenMap = (Map<Long, ReportPO>)children.stream().collect(Collectors.toMap(ReportPO::getId, Function.identity()));
    dragPO.setParentId(dragDTO.getParentId());
    int order = 1;
    for (Long id : dragDTO.getSortIds()) {
      ReportPO updatePO;
      if (id.equals(dragDTO.getDragId())) {
        updatePO = dragPO;
      } else if (childrenMap.containsKey(id)) {
        updatePO = childrenMap.get(id);
      } else {
        continue;
      } 
      updatePO.setOrder(Integer.valueOf(order++));
      updatePO.setUpdateTime(new Date());
      this.reportDao.moveReport(updatePO);
    } 
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  private boolean isNameDuplicate(SaveReportDTO param) {
    ReportPO report = this.reportDao.selectReportByParentIdAndTypeAndName(param);
    return (report != null && !Objects.equals(report.getId(), param.getId()));
  }
  
  private Integer getNextOrder(SaveReportDTO param) {
    Integer maxOrder = this.reportDao.getMaxOrder(param);
    return Integer.valueOf((maxOrder == null) ? 1 : (maxOrder.intValue() + 1));
  }
}
